From c3338ec1a4fff96022c99671c73ee9841871f6e8 Mon Sep 17 00:00:00 2001
From: Takashi Yano <takashi.yano@nifty.ne.jp>
Date: Sat, 2 Jul 2022 12:08:51 +0900
Subject: [PATCH] Cygwin: clipboard: Add workaround for setting clipboard
 failure.

- OpenClipboard() just after CloseClipboard() has a high probabolity
  of failure. Due to this, /dev/clipboard sometimes fails to set
  CF_UNICODETEXT data. This patch add a workaround for this issue.
---
 winsup/cygwin/fhandler_clipboard.cc | 22 +++++++++++++++++-----
 1 file changed, 17 insertions(+), 5 deletions(-)

diff --git a/winsup/cygwin/fhandler_clipboard.cc b/winsup/cygwin/fhandler_clipboard.cc
index 05f54ffb3..b0cdf485e 100644
--- a/winsup/cygwin/fhandler_clipboard.cc
+++ b/winsup/cygwin/fhandler_clipboard.cc
@@ -19,6 +19,18 @@ details. */
 #include "child_info.h"
 #include <sys/clipboard.h>
 
+/* Opening clipboard immediately after CloseClipboard() has
+   high probability of failure. Therefore use retry-loop. */
+static inline bool
+open_clipboard ()
+{
+  const int max_retry = 10;
+  for (int i = 0; i < max_retry; i++)
+    if (OpenClipboard (NULL))
+      return true;
+  return false;
+}
+
 /*
  * Robert Collins:
  * FIXME: should we use GetClipboardSequenceNumber to tell if the clipboard has
@@ -29,7 +41,7 @@ fhandler_dev_clipboard::fhandler_dev_clipboard ()
   : fhandler_base (), pos (0), membuffer (NULL), msize (0)
 {
   /* FIXME: check for errors and loop until we can open the clipboard */
-  OpenClipboard (NULL);
+  open_clipboard ();
   cygnativeformat = RegisterClipboardFormatW (CYGWIN_NATIVE);
   CloseClipboard ();
 }
@@ -53,7 +65,7 @@ fhandler_dev_clipboard::set_clipboard (const void *buf, size_t len)
 {
   HGLOBAL hmem;
   /* Native CYGWIN format */
-  if (OpenClipboard (NULL))
+  if (open_clipboard ())
     {
       cygcb_t *clipbuf;
 
@@ -99,7 +111,7 @@ fhandler_dev_clipboard::set_clipboard (const void *buf, size_t len)
       set_errno (EILSEQ);
       return -1;
     }
-  if (OpenClipboard (NULL))
+  if (open_clipboard ())
     {
       PWCHAR clipbuf;
 
@@ -168,7 +180,7 @@ fhandler_dev_clipboard::fstat (struct stat *buf)
   buf->st_ctim.tv_nsec = 0L;
   buf->st_birthtim = buf->st_atim = buf->st_mtim = buf->st_ctim;
 
-  if (OpenClipboard (NULL))
+  if (open_clipboard ())
     {
       UINT formatlist[1] = { cygnativeformat };
       int format;
@@ -207,7 +219,7 @@ fhandler_dev_clipboard::read (void *ptr, size_t& len)
   LPVOID cb_data;
   int rach;
 
-  if (!OpenClipboard (NULL))
+  if (!open_clipboard ())
     {
       len = 0;
       return;
-- 
2.36.1

